@@ -55,7 +55,7 @@ function CSPHeaders($view, $nonce) {
       // fall through
     default:
       // Enforce script-src on pages where inline scripts and event handlers have been fixed.
-      header("Content-Security-Policy: script-src 'self' 'nonce-$nonce' $additionalScriptSrc".
+      header("Content-Security-Policy: object-src 'self'; script-src 'self' 'nonce-$nonce' $additionalScriptSrc".
         (ZM_CSP_REPORT_URI ? '; report-uri '.ZM_CSP_REPORT_URI : '' )
       );
       break;
@@ -369,10 +369,16 @@ function getEventDefaultVideoPath($event) {
 
 function deletePath( $path ) {
   ZM\Debug('Deleting '.$path);
-  if ( is_dir($path) ) {
-    system(escapeshellcmd('rm -rf '.$path));
-  } else if ( file_exists($path) ) {
-    unlink($path);
+  if (is_link($path)) {
+    if (!unlink($path)) ZM\Debug("Failed to unlink $path");
+  } else if (is_dir($path)) {
+    if (false === ($output = system('rm -rf "'.escapeshellcmd($path).'"'))) {
+      ZM\Warning('Failed doing rm -rf "'.escapeshellcmd($path).'"');
+    }
+  } else if (file_exists($path)) {
+    if (!unlink($path)) ZM\Debug("Failed to delete $path");
+  } else {
+    ZM\Warning("Path $path does not exist in deletePath()");
   }
 }
 
@@ -1041,6 +1047,8 @@ function parseFilter(&$filter, $saveToSession=false, $querySep='&amp;') {
 
   $Filter = ZM\Filter::parse($filter, $querySep);
 
+  if (isset($filter['Id']))
+    $filter['Id'] = validCardinal($filter['Id']);
   $filter['sql'] = $Filter->sql();
   $filter['querystring'] = $Filter->querystring('filter', $querySep);
   $filter['hidden_fields'] = $Filter->hidden_fields();
@@ -1909,8 +1917,13 @@ function generateConnKey() {
 }
 
 function detaintPath($path) {
+
+  // Strip out :// because php:// is a way to inject code apparently
+  $path = str_replace('://', '', $path);
   // Remove any absolute paths, or relative ones that want to go up
-  $path = str_replace('../', '', $path);
+  do {
+    $path = str_replace('../', '', $path, $count);
+  } while($count);
   $path = ltrim($path, '/');
   return $path;
 }
@@ -2029,7 +2042,7 @@ function getStreamHTML($monitor, $options = array()) {
         #$options['width'] = $monitor->ViewWidth().'px';
         #$options['height'] = $monitor->ViewHeight().'px';
       }
-    } else {
+    } else if ($options['width'] != 'auto') {
       #ZM\Warning("Have width ".$options['width']);
       if ( preg_match('/^(\d+)px$/', $options['width'], $matches) ) {
         $scale = intval(100*$matches[1]/$monitor->ViewWidth());
@@ -2396,4 +2409,8 @@ function i18n() {
 
   return implode('-', $string);
 }
+
+function check_datetime($x) {
+  return (date('Y-m-d H:i:s', strtotime($x)) == $x);
+}
 ?>
