@@ -118,6 +118,10 @@ if (!is_dir("skins/$skin/css/$css")) {
     if (!in_array($css, $css_skins)) {
       ZM\Error("Invalid skin css '$css' setting to " . $css_skins[0]);
       $css = $css_skins[0];
+      if (isset($_COOKIE['zmCSS'])) {
+        unset($_COOKIE['zmCSS']);
+        setcookie('zmCSS', '', time() - 3600);
+      }
     } else {
       $css = '';
     }
@@ -196,7 +200,7 @@ foreach ( getSkinIncludes('skin.php') as $includeFile ) {
 if (isset($_POST['action'])) {
   # Actions can only be performed on POST because we don't check csrf on GETs.
   $action = detaintPath($_POST['action']);
-} else if (isset($_REQUEST['action']) and $_REQUEST['action']) {
+} else if (isset($_REQUEST['action']) and $_REQUEST['action'] and empty($_REQUEST['request'])) {
   ZM\Error('actions can no longer be performed without POST. Requested: '.$_REQUEST['action']. ' for ' .$view);
 }
 
@@ -225,15 +229,6 @@ if (
   csrf_check();
 }
 
-# Need to include actions because it does auth
-if ( $action and $view and !$request ) {
-  if ( file_exists('includes/actions/'.$view.'.php') ) {
-    require_once('includes/actions/'.$view.'.php');
-  } else {
-    ZM\Warning("No includes/actions/$view.php for action $action");
-  }
-}
-
 # If I put this here, it protects all views and popups, but it has to go after actions.php because actions.php does the actual logging in.
 if ( ZM_OPT_USE_AUTH and (!isset($user)) and ($view != 'login') and ($view != 'none') ) {
   if ($request) {
@@ -252,6 +247,15 @@ if ( ZM_OPT_USE_AUTH and (!isset($user)) and ($view != 'login') and ($view != 'n
   $request = null;
 }
 
+# Need to include actions because it does auth
+if ( $action and $view and !$request ) {
+  if ( file_exists('includes/actions/'.$view.'.php') ) {
+    require_once('includes/actions/'.$view.'.php');
+  } else {
+    ZM\Debug("No includes/actions/$view.php for action $action");
+  }
+}
+
 if ( isset($_REQUEST['redirect']) ) {
   $redirect = '?view='.detaintPath($_REQUEST['redirect']);
 }
@@ -277,8 +281,11 @@ if ( $includeFiles = getSkinIncludes('views/'.$view.'.php', true, true) ) {
   ob_start();
   CSPHeaders($view, $cspNonce);
   foreach ( $includeFiles as $includeFile ) {
-    if (!file_exists($includeFile))
-      ZM\Fatal("View '$view' does not exist");
+    if (!file_exists($includeFile)) {
+      ZM\Error("View '$view' does not exist, redirecting to console");
+      header('Location: ?view=console');
+      return;
+    }
     require_once $includeFile;
   }
   // If the view overrides $view to 'error', and the user is not logged in, then the
