@@ -111,15 +111,15 @@ echo output_cache_busted_stylesheet_links(array(
   'css/reset.css',
   'css/font-awesome.min.css',
   'css/bootstrap.min.css',
-  'css/bootstrap-table.min.css',
-  'css/bootstrap-table-page-jump-to.min.css',
+  'skins/classic/js/bootstrap-table-1.23.5/bootstrap-table.min.css',
+  'skins/classic/js/bootstrap-table-1.23.5/extensions/page-jump-to/bootstrap-table-page-jump-to.min.css',
 ));
 
 echo output_link_if_exists(array(
   'css/base/skin.css',
   'css/base/views/'.$basename.'.css',
   'js/dateTimePicker/jquery-ui-timepicker-addon.css',
-  'js/jquery-ui-1.12.1/jquery-ui.structure.min.css',
+  'js/jquery-ui-1.13.2/jquery-ui.structure.min.css',
 ));
 if ( $css != 'base' )
   echo output_link_if_exists(array(
@@ -128,7 +128,7 @@ if ( $css != 'base' )
     'css/'.$css.'/jquery-ui-theme.css',
   ));
 ?>
-  <link rel="stylesheet" href="skins/classic/js/jquery-ui-1.12.1/jquery-ui.theme.min.css" type="text/css"/>
+  <link rel="stylesheet" href="skins/classic/js/jquery-ui-1.13.2/jquery-ui.theme.min.css" type="text/css"/>
   <?php #Chosen can't be cache-busted because it loads sprites by relative path ?>
   <link rel="stylesheet" href="skins/classic/js/chosen/chosen.min.css" type="text/css"/>
 <?php
@@ -169,7 +169,7 @@ function getBodyTopHTML() {
 ';
   global $error_message;
   if ( $error_message ) {
-   echo '<div class="error">'.$error_message.'</div>';
+   echo '<div id="error">'.$error_message.'</div>';
   }
 } // end function getBodyTopHTML
 
@@ -203,7 +203,7 @@ function getNormalNavBarHTML($running, $user, $bandwidth_options, $view, $skin)
   $status = runtimeStatus($running);
 
 ?>
-<div class="fixed-top container-fluid p-0">
+<div class="container-fluid p-0">
   <nav class="navbar navbar-expand-md navbar-dark bg-dark justify-content-center flex-row">
 
     <div class="navbar-brand justify-content-start align-self-start">
@@ -386,7 +386,7 @@ function getSysLoadHTML() {
 
   $result .= '<li id="getSysLoadHTML" class="Load nav-item mx-2">'.PHP_EOL;
   $result .= '<i class="material-icons md-18">trending_up</i>'.PHP_EOL;
-  $result .= '&nbsp;'.translate('Load').': '.getLoad().PHP_EOL;
+  $result .= '&nbsp;'.translate('Load').': '.number_format(getLoad(), 2, '.', '').PHP_EOL;
   $result .= '</li>'.PHP_EOL;
   
   return $result;
@@ -414,8 +414,7 @@ function getStorageHTML() {
   $result = '';
   if ( !canView('System') ) return $result;
 
-  $func = function($S) {
-    $class = '';
+  $func = function($S, $class='') {
     if ( $S->disk_usage_percent() > 98 ) {
       $class = 'text-danger';
     } else if ( $S->disk_usage_percent() > 90 ) {
@@ -423,7 +422,7 @@ function getStorageHTML() {
     }
     $title = human_filesize($S->disk_used_space()) . ' of ' . human_filesize($S->disk_total_space()). 
       ( ( $S->disk_used_space() != $S->event_disk_space() ) ? ' ' .human_filesize($S->event_disk_space()) . ' used by events' : '' );
-    return '<a class="dropdown-item '.$class.'" title="'.$title.'" href="?view=options&amp;tab=storage">'.$S->Name() . ': ' . $S->disk_usage_percent().'%' . '</a>';
+    return '<a class="'.$class.'" title="'.$title.'" href="?view=options&amp;tab=storage">'.validHtmlStr($S->Name()) . ': ' . $S->disk_usage_percent().'%' . '</a>';
   };
 
   $storage_areas = ZM\Storage::find(array('Enabled'=>true));
@@ -443,15 +442,23 @@ function getStorageHTML() {
     $class = 'text-warning'; 
   }
   
-  $result .= '<li id="getStorageHTML" class="nav-item dropdown mx-2">'.PHP_EOL;
-  $result .= '<a class="dropdown-toggle mr-2 '.$class.'" href="#" id="dropdown_storage" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"><i class="material-icons md-18 mr-1">folder_shared</i>Storage</a>'.PHP_EOL;
-  $result .= '<div class="dropdown-menu" aria-labelledby="dropdown_storage">'.PHP_EOL;
-  
-  foreach ( $storage_areas as $area ) {  
-    $result .= $func($area).PHP_EOL;
-  } 
-  $result .= '</div>'.PHP_EOL;
-  $result .= '</li>'.PHP_EOL;
+  if (count($storage_areas) <= 2) {
+    $result .= '<li id="getStorageHTML" class="nav-item mx-2">'.PHP_EOL;
+    foreach ( $storage_areas as $area ) {  
+      $result .= $func($area).PHP_EOL;
+    } 
+    $result .= '</li>'.PHP_EOL;
+  } else {
+    $result .= '<li id="getStorageHTML" class="nav-item dropdown mx-2">'.PHP_EOL;
+    $result .= '<a class="dropdown-toggle mr-2 '.$class.'" href="#" id="dropdown_storage" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"><i class="material-icons md-18 mr-1">folder_shared</i>Storage</a>'.PHP_EOL;
+    $result .= '<div class="dropdown-menu" aria-labelledby="dropdown_storage">'.PHP_EOL;
+    
+    foreach ( $storage_areas as $area ) {  
+      $result .= $func($area, 'dropdown-item ').PHP_EOL;
+    } 
+    $result .= '</div>'.PHP_EOL;
+    $result .= '</li>'.PHP_EOL;
+  }
   
   return $result;
 }
@@ -499,21 +506,22 @@ function getBandwidthHTML($bandwidth_options, $user) {
     }
   }
 
-  $result = '<li id="getBandwidthHTML" class="nav-item dropdown mx-2">'.PHP_EOL;
-  $result .= '<a class="dropdown-toggle mr-2" href="#" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" id="dropdown_bandwidth"><i class="material-icons md-18 mr-1">network_check</i>'.translate($bandwidth_options[$_COOKIE['zmBandwidth']]).'</a>'.PHP_EOL;
+  $result = '';
+  if (count($bandwidth_options) > 1) {
+    $result .= '<li id="getBandwidthHTML" class="nav-item dropdown mx-2">'.PHP_EOL;
+    $result .= '<a class="dropdown-toggle mr-2" href="#" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" id="dropdown_bandwidth"><i class="material-icons md-18 mr-1">network_check</i>'.translate($bandwidth_options[$_COOKIE['zmBandwidth']]).'</a>'.PHP_EOL;
 
-  $result .= '<div class="dropdown-menu" aria-labelledby="dropdown_bandwidth">'.PHP_EOL;  
-  if ( count($bandwidth_options) > 1 ) {
+    $result .= '<div class="dropdown-menu" aria-labelledby="dropdown_bandwidth">'.PHP_EOL;  
     if ( isset($bandwidth_options['high']) )
       $result .= '<a data-pdsa-dropdown-val="high" class="dropdown-item bwselect" href="#">' .translate('High'). '</a>'.PHP_EOL;
     if ( isset($bandwidth_options['medium']) )
       $result .= '<a data-pdsa-dropdown-val="medium" class="dropdown-item bwselect" href="#">' .translate('Medium'). '</a>'.PHP_EOL;
     # low is theoretically always available
     $result .= '<a data-pdsa-dropdown-val="low" class="dropdown-item bwselect" href="#">' .translate('Low'). '</a>'.PHP_EOL;    
-  }
-  $result .= '</div>'.PHP_EOL;
+    $result .= '</div>'.PHP_EOL;
 
-  $result .= '</li>'.PHP_EOL;
+    $result .= '</li>'.PHP_EOL;
+  }
     
   return $result;
 }
@@ -754,7 +762,7 @@ function getAccountCircleHTML($skin, $user=null) {
   
   if ( ZM_OPT_USE_AUTH and $user ) {
     $result .= '<li id="getAccountCircleHTML" class="navbar-text navbar-nav mr-2">'.PHP_EOL;
-    $result .= makeLink('#', '<i class="material-icons">account_circle</i> '.  $user['Username'],
+    $result .= makeLink('#', '<i class="material-icons">account_circle</i> '.  validHtmlStr($user['Username']),
       (ZM_AUTH_TYPE == 'builtin'), 'id="logoutButton" data-toggle="modal" data-target="#modalLogout" data-backdrop="false"' ).PHP_EOL;
     $result .= '</li>'.PHP_EOL;
   }
@@ -766,25 +774,23 @@ function getAccountCircleHTML($skin, $user=null) {
 function getStatusBtnHTML($status) {
   $result = '';
   
-  if ( canEdit('System') ) {
-    //$result .= '<li class="nav-item dropdown">'.PHP_EOL;
+  if (canEdit('System')) {
     $result .= '<li id="getStatusBtnHTML">'.PHP_EOL;
     $result .= '<button type="button" class="btn btn-default navbar-btn" id="stateModalBtn">' .$status. '</button>'.PHP_EOL;
     $result .= '</li>'.PHP_EOL;
-    //$result .= '</li>'.PHP_EOL;
 
-    if ( ZM_SYSTEM_SHUTDOWN ) {
-      $result .= '<li class="navbar-text pr-2 align-self-center">'.PHP_EOL;
-      $result .= '<button class="btn btn-outline" data-on-click="getShutdownModal" data-toggle="tooltip" data-placement="top" title="' .translate("Shutdown"). '" ><i class="material-icons md-18">power_settings_new</i></button>'.PHP_EOL;
+    if (ZM_SYSTEM_SHUTDOWN) {
+      $result .= '<li class="pr-2">'.PHP_EOL;
+      $result .= '<button id="shutdownButton" class="btn btn-default navbar-btn" data-on-click="getShutdownModal" data-toggle="tooltip" data-placement="top" title="' .translate('Shutdown'). '"><i class="material-icons md-18">power_settings_new</i></button>'.PHP_EOL;
       $result .= '</li>'.PHP_EOL;
      } 
 
-  } else if ( canView('System') ) {
+  } else if (canView('System')) {
     $result .= '<li id="getStatusBtnHTML" class="navbar-text">'.PHP_EOL;
     $result .= $status.PHP_EOL;
     $result .= '</li>'.PHP_EOL;
   }
-  
+
   return $result;
 }
 
@@ -884,23 +890,23 @@ function xhtmlFooter() {
   $viewJsPhpFile = getSkinFile('views/js/'.$basename.'.js.php');
 ?>
   <script src="<?php echo cache_bust('skins/'.$skin.'/js/jquery.min.js'); ?>"></script>
-  <script src="skins/<?php echo $skin; ?>/js/jquery-ui-1.12.1/jquery-ui.min.js"></script>
-  <script src="<?php echo cache_bust('skins/'.$skin.'/js/bootstrap.min.js'); ?>"></script>
+  <script src="skins/<?php echo $skin; ?>/js/jquery-ui-1.13.2/jquery-ui.min.js"></script>
+  <script src="<?php echo cache_bust('js/ajaxQueue.js') ?>"></script>
+  <script src="<?php echo 'skins/'.$skin.'/js/bootstrap.min.js' ?>"></script>
 <?php echo output_script_if_exists(array(
   'js/tableExport.min.js',
-  'js/bootstrap-table.min.js',
-  'js/bootstrap-table-locale-all.min.js',
-  'js/bootstrap-table-export.min.js',
-  'js/bootstrap-table-page-jump-to.min.js',
-  'js/bootstrap-table-cookie.min.js',
-  'js/bootstrap-table-toolbar.min.js',
-  'js/bootstrap-table-auto-refresh.min.js',
+  'js/bootstrap-table-1.23.5/bootstrap-table.min.js',
+  'js/bootstrap-table-1.23.5/bootstrap-table-locale-all.min.js',
+  'js/bootstrap-table-1.23.5/extensions/export/bootstrap-table-export.min.js',
+  'js/bootstrap-table-1.23.5/extensions/page-table-to/bootstrap-table-page-jump-to.min.js',
+  'js/bootstrap-table-1.23.5/extensions/cookie/bootstrap-table-cookie.min.js',
+  'js/bootstrap-table-1.23.5/extensions/toolbar/bootstrap-table-toolbar.min.js',
+  'js/bootstrap-table-1.23.5/extensions/auto-refresh/bootstrap-table-auto-refresh.min.js',
   'js/chosen/chosen.jquery.min.js',
   'js/dateTimePicker/jquery-ui-timepicker-addon.js',
   'js/Server.js',
 ), true );
 ?>
-  <script nonce="<?php echo $cspNonce; ?>">var $j = jQuery.noConflict();</script>
 <?php
   if ( $view == 'event' ) {
 ?>
@@ -914,7 +920,7 @@ function xhtmlFooter() {
   <script src="skins/<?php echo $skin ?>/js/moment.min.js"></script>
 <?php
 ?>
-  <script nonce="<?php echo $cspNonce; ?>">
+  <script nonce="<?php echo $cspNonce; ?>">var $j = jQuery.noConflict();
 <?php
   if ( $skinJsPhpFile ) {
     require_once( $skinJsPhpFile );
